# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/view_index.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(800, 600)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.scrollArea = QtWidgets.QScrollArea(self.centralwidget)
        self.scrollArea.setHorizontalScrollBarPolicy(QtCore.Qt.ScrollBarAsNeeded)
        self.scrollArea.setWidgetResizable(True)
        self.scrollArea.setObjectName("scrollArea")
        self.scrollAreaWidgetContents = QtWidgets.QWidget()
        self.scrollAreaWidgetContents.setGeometry(QtCore.QRect(0, 0, 374, 508))
        self.scrollAreaWidgetContents.setObjectName("scrollAreaWidgetContents")
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout(self.scrollAreaWidgetContents)
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label_Pict = QtWidgets.QLabel(self.scrollAreaWidgetContents)
        self.label_Pict.setText("")
        self.label_Pict.setObjectName("label_Pict")
        self.horizontalLayout_2.addWidget(self.label_Pict)
        self.scrollArea.setWidget(self.scrollAreaWidgetContents)
        self.horizontalLayout.addWidget(self.scrollArea)
        self.frame_2 = QtWidgets.QFrame(self.centralwidget)
        self.frame_2.setMaximumSize(QtCore.QSize(500, 16777215))
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.verticalLayout = QtWidgets.QVBoxLayout(self.frame_2)
        self.verticalLayout.setObjectName("verticalLayout")
        self.listView_Pairs = QtWidgets.QListView(self.frame_2)
        self.listView_Pairs.setObjectName("listView_Pairs")
        self.verticalLayout.addWidget(self.listView_Pairs)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label = QtWidgets.QLabel(self.frame_2)
        self.label.setMaximumSize(QtCore.QSize(100, 16777215))
        self.label.setObjectName("label")
        self.horizontalLayout_3.addWidget(self.label)
        self.spinBox_Scale = QtWidgets.QSpinBox(self.frame_2)
        self.spinBox_Scale.setMinimum(1)
        self.spinBox_Scale.setMaximum(10)
        self.spinBox_Scale.setObjectName("spinBox_Scale")
        self.horizontalLayout_3.addWidget(self.spinBox_Scale)
        self.verticalLayout.addLayout(self.horizontalLayout_3)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_2 = QtWidgets.QLabel(self.frame_2)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_4.addWidget(self.label_2)
        self.edit_MinBirth = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MinBirth.setObjectName("edit_MinBirth")
        self.horizontalLayout_4.addWidget(self.edit_MinBirth)
        self.verticalLayout.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_3 = QtWidgets.QLabel(self.frame_2)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_5.addWidget(self.label_3)
        self.edit_MaxBirth = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MaxBirth.setObjectName("edit_MaxBirth")
        self.horizontalLayout_5.addWidget(self.edit_MaxBirth)
        self.verticalLayout.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_4 = QtWidgets.QLabel(self.frame_2)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_6.addWidget(self.label_4)
        self.edit_MinDeath = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MinDeath.setObjectName("edit_MinDeath")
        self.horizontalLayout_6.addWidget(self.edit_MinDeath)
        self.verticalLayout.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.label_5 = QtWidgets.QLabel(self.frame_2)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_7.addWidget(self.label_5)
        self.edit_MaxDeath = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MaxDeath.setObjectName("edit_MaxDeath")
        self.horizontalLayout_7.addWidget(self.edit_MaxDeath)
        self.verticalLayout.addLayout(self.horizontalLayout_7)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.label_6 = QtWidgets.QLabel(self.frame_2)
        self.label_6.setObjectName("label_6")
        self.horizontalLayout_8.addWidget(self.label_6)
        self.edit_MinLifetime = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MinLifetime.setObjectName("edit_MinLifetime")
        self.horizontalLayout_8.addWidget(self.edit_MinLifetime)
        self.verticalLayout.addLayout(self.horizontalLayout_8)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_7 = QtWidgets.QLabel(self.frame_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_9.addWidget(self.label_7)
        self.edit_MaxLifetime = QtWidgets.QLineEdit(self.frame_2)
        self.edit_MaxLifetime.setObjectName("edit_MaxLifetime")
        self.horizontalLayout_9.addWidget(self.edit_MaxLifetime)
        self.verticalLayout.addLayout(self.horizontalLayout_9)
        self.button_Filter = QtWidgets.QPushButton(self.frame_2)
        self.button_Filter.setObjectName("button_Filter")
        self.verticalLayout.addWidget(self.button_Filter)
        self.horizontalLayout.addWidget(self.frame_2)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 800, 35))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "View index for a picture"))
        self.label.setText(_translate("MainWindow", "Zoom:"))
        self.label_2.setText(_translate("MainWindow", "min birth:"))
        self.label_3.setText(_translate("MainWindow", "max birth:"))
        self.label_4.setText(_translate("MainWindow", "min death:"))
        self.label_5.setText(_translate("MainWindow", "max death:"))
        self.label_6.setText(_translate("MainWindow", "min lifetime:"))
        self.label_7.setText(_translate("MainWindow", "max lifetime:"))
        self.button_Filter.setText(_translate("MainWindow", "Apply Filter"))
        self.menuFile.setTitle(_translate("MainWindow", "File"))
        self.actionQuit.setText(_translate("MainWindow", "Quit"))
        self.actionQuit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
