#ifndef HERA_COMMON_HASH_COMBINE_H
#define HERA_COMMON_HASH_COMBINE_H

namespace hera {
    template <class T>
    inline void hash_combine(std::size_t& seed, const T& v)
    {
        std::hash<T> hasher;
        seed ^= hasher(v) + 0x9e3779b9 + (seed << 6) + (seed >> 2);
    }
}

#endif

