// -*- mode: c++ -*-
#ifndef HOMCLOUD_COMMON_H
#define HOMCLOUD_COMMON_H

#define PY_SSIZE_T_CLEAN
#include "Python.h"
#include "structmember.h"

#pragma GCC diagnostic ignored "-Wwrite-strings"

#endif // HOMCLOUD_COMMON_H
