# -*- coding: utf-8 -*-

# Form implementation generated from reading ui file 'ui/plot_PD_GUI.ui'
#
# Created by: PyQt5 UI code generator 5.15.6
#
# WARNING: Any manual changes made to this file will be lost when pyuic5 is
# run again.  Do not edit this file unless you know what you are doing.


from PyQt5 import QtCore, QtGui, QtWidgets


class Ui_MainWindow(object):
    def setupUi(self, MainWindow):
        MainWindow.setObjectName("MainWindow")
        MainWindow.resize(876, 1038)
        self.centralwidget = QtWidgets.QWidget(MainWindow)
        self.centralwidget.setObjectName("centralwidget")
        self.horizontalLayout = QtWidgets.QHBoxLayout(self.centralwidget)
        self.horizontalLayout.setObjectName("horizontalLayout")
        self.mplvl = QtWidgets.QVBoxLayout()
        self.mplvl.setObjectName("mplvl")
        self.horizontalLayout.addLayout(self.mplvl)
        self.frame = QtWidgets.QFrame(self.centralwidget)
        self.frame.setMaximumSize(QtCore.QSize(300, 16777215))
        self.frame.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame.setObjectName("frame")
        self.verticalLayout_2 = QtWidgets.QVBoxLayout(self.frame)
        self.verticalLayout_2.setObjectName("verticalLayout_2")
        spacerItem = QtWidgets.QSpacerItem(20, 40, QtWidgets.QSizePolicy.Minimum, QtWidgets.QSizePolicy.Expanding)
        self.verticalLayout_2.addItem(spacerItem)
        self.frame_optimal_volume = QtWidgets.QFrame(self.frame)
        self.frame_optimal_volume.setMinimumSize(QtCore.QSize(0, 0))
        self.frame_optimal_volume.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_optimal_volume.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_optimal_volume.setObjectName("frame_optimal_volume")
        self.gridLayout_5 = QtWidgets.QGridLayout(self.frame_optimal_volume)
        self.gridLayout_5.setObjectName("gridLayout_5")
        self.horizontalLayout_13 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_13.setObjectName("horizontalLayout_13")
        self.label_13 = QtWidgets.QLabel(self.frame_optimal_volume)
        self.label_13.setObjectName("label_13")
        self.horizontalLayout_13.addWidget(self.label_13)
        self.lineEdit_optimal_volume_stable_volume = QtWidgets.QLineEdit(self.frame_optimal_volume)
        self.lineEdit_optimal_volume_stable_volume.setObjectName("lineEdit_optimal_volume_stable_volume")
        self.horizontalLayout_13.addWidget(self.lineEdit_optimal_volume_stable_volume)
        self.gridLayout_5.addLayout(self.horizontalLayout_13, 7, 0, 1, 2)
        self.radioButton_eigenvolume_tightened = QtWidgets.QRadioButton(self.frame_optimal_volume)
        self.radioButton_eigenvolume_tightened.setObjectName("radioButton_eigenvolume_tightened")
        self.gridLayout_5.addWidget(self.radioButton_eigenvolume_tightened, 5, 1, 1, 1)
        self.radioButton_eigenvolume_tightened_sub = QtWidgets.QRadioButton(self.frame_optimal_volume)
        self.radioButton_eigenvolume_tightened_sub.setObjectName("radioButton_eigenvolume_tightened_sub")
        self.gridLayout_5.addWidget(self.radioButton_eigenvolume_tightened_sub, 6, 1, 1, 1)
        self.horizontalLayout_12 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_12.setObjectName("horizontalLayout_12")
        self.label_12 = QtWidgets.QLabel(self.frame_optimal_volume)
        self.label_12.setObjectName("label_12")
        self.horizontalLayout_12.addWidget(self.label_12)
        self.lineEdit_optimal_volume_retry = QtWidgets.QLineEdit(self.frame_optimal_volume)
        self.lineEdit_optimal_volume_retry.setMaximumSize(QtCore.QSize(70, 16777215))
        self.lineEdit_optimal_volume_retry.setObjectName("lineEdit_optimal_volume_retry")
        self.horizontalLayout_12.addWidget(self.lineEdit_optimal_volume_retry)
        self.label_11 = QtWidgets.QLabel(self.frame_optimal_volume)
        self.label_11.setObjectName("label_11")
        self.horizontalLayout_12.addWidget(self.label_11)
        self.lineEdit_optimal_volume_cutoff_radius = QtWidgets.QLineEdit(self.frame_optimal_volume)
        self.lineEdit_optimal_volume_cutoff_radius.setObjectName("lineEdit_optimal_volume_cutoff_radius")
        self.horizontalLayout_12.addWidget(self.lineEdit_optimal_volume_cutoff_radius)
        self.gridLayout_5.addLayout(self.horizontalLayout_12, 3, 0, 1, 2)
        self.radioButton_eigenvolume_owned = QtWidgets.QRadioButton(self.frame_optimal_volume)
        self.radioButton_eigenvolume_owned.setObjectName("radioButton_eigenvolume_owned")
        self.gridLayout_5.addWidget(self.radioButton_eigenvolume_owned, 6, 0, 1, 1)
        self.radioButton_eigenvolume_none = QtWidgets.QRadioButton(self.frame_optimal_volume)
        self.radioButton_eigenvolume_none.setChecked(True)
        self.radioButton_eigenvolume_none.setObjectName("radioButton_eigenvolume_none")
        self.gridLayout_5.addWidget(self.radioButton_eigenvolume_none, 5, 0, 1, 1)
        self.checkBox_optimal_volume_query_children = QtWidgets.QCheckBox(self.frame_optimal_volume)
        self.checkBox_optimal_volume_query_children.setObjectName("checkBox_optimal_volume_query_children")
        self.gridLayout_5.addWidget(self.checkBox_optimal_volume_query_children, 1, 0, 1, 2)
        self.checkBox_optimal_volume_on = QtWidgets.QCheckBox(self.frame_optimal_volume)
        self.checkBox_optimal_volume_on.setObjectName("checkBox_optimal_volume_on")
        self.gridLayout_5.addWidget(self.checkBox_optimal_volume_on, 0, 0, 1, 2)
        self.horizontalLayout_11 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_11.setObjectName("horizontalLayout_11")
        self.checkBox_optimal_volume_show_children = QtWidgets.QCheckBox(self.frame_optimal_volume)
        self.checkBox_optimal_volume_show_children.setChecked(True)
        self.checkBox_optimal_volume_show_children.setObjectName("checkBox_optimal_volume_show_children")
        self.horizontalLayout_11.addWidget(self.checkBox_optimal_volume_show_children)
        self.checkBox_optimal_volume_show_volume = QtWidgets.QCheckBox(self.frame_optimal_volume)
        self.checkBox_optimal_volume_show_volume.setChecked(True)
        self.checkBox_optimal_volume_show_volume.setObjectName("checkBox_optimal_volume_show_volume")
        self.horizontalLayout_11.addWidget(self.checkBox_optimal_volume_show_volume)
        self.gridLayout_5.addLayout(self.horizontalLayout_11, 2, 0, 1, 2)
        self.label_16 = QtWidgets.QLabel(self.frame_optimal_volume)
        self.label_16.setObjectName("label_16")
        self.gridLayout_5.addWidget(self.label_16, 4, 0, 1, 1)
        self.verticalLayout_2.addWidget(self.frame_optimal_volume)
        self.frame_phtree = QtWidgets.QFrame(self.frame)
        self.frame_phtree.setFrameShape(QtWidgets.QFrame.Panel)
        self.frame_phtree.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_phtree.setObjectName("frame_phtree")
        self.gridLayout = QtWidgets.QGridLayout(self.frame_phtree)
        self.gridLayout.setObjectName("gridLayout")
        self.button_show_descendants = QtWidgets.QPushButton(self.frame_phtree)
        self.button_show_descendants.setObjectName("button_show_descendants")
        self.gridLayout.addWidget(self.button_show_descendants, 4, 0, 1, 2)
        self.checkBox_query_volumes_in_rectangle = QtWidgets.QCheckBox(self.frame_phtree)
        self.checkBox_query_volumes_in_rectangle.setObjectName("checkBox_query_volumes_in_rectangle")
        self.gridLayout.addWidget(self.checkBox_query_volumes_in_rectangle, 5, 0, 1, 2)
        self.checkBox_show_descendants = QtWidgets.QCheckBox(self.frame_phtree)
        self.checkBox_show_descendants.setChecked(True)
        self.checkBox_show_descendants.setObjectName("checkBox_show_descendants")
        self.gridLayout.addWidget(self.checkBox_show_descendants, 2, 1, 1, 1)
        self.checkBox_show_ancestors = QtWidgets.QCheckBox(self.frame_phtree)
        self.checkBox_show_ancestors.setChecked(True)
        self.checkBox_show_ancestors.setObjectName("checkBox_show_ancestors")
        self.gridLayout.addWidget(self.checkBox_show_ancestors, 2, 0, 1, 1)
        self.checkBox_query_phtree = QtWidgets.QCheckBox(self.frame_phtree)
        self.checkBox_query_phtree.setObjectName("checkBox_query_phtree")
        self.gridLayout.addWidget(self.checkBox_query_phtree, 1, 0, 1, 2)
        self.verticalLayout_2.addWidget(self.frame_phtree)
        self.frame_3 = QtWidgets.QFrame(self.frame)
        self.frame_3.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_3.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_3.setObjectName("frame_3")
        self.horizontalLayout_10 = QtWidgets.QHBoxLayout(self.frame_3)
        self.horizontalLayout_10.setObjectName("horizontalLayout_10")
        self.label_10 = QtWidgets.QLabel(self.frame_3)
        self.label_10.setMinimumSize(QtCore.QSize(100, 0))
        self.label_10.setObjectName("label_10")
        self.horizontalLayout_10.addWidget(self.label_10)
        self.radioButton_normalplot = QtWidgets.QRadioButton(self.frame_3)
        self.radioButton_normalplot.setChecked(True)
        self.radioButton_normalplot.setObjectName("radioButton_normalplot")
        self.horizontalLayout_10.addWidget(self.radioButton_normalplot)
        self.radioButton_contourplot = QtWidgets.QRadioButton(self.frame_3)
        self.radioButton_contourplot.setObjectName("radioButton_contourplot")
        self.horizontalLayout_10.addWidget(self.radioButton_contourplot)
        self.verticalLayout_2.addWidget(self.frame_3)
        self.frame_2 = QtWidgets.QFrame(self.frame)
        self.frame_2.setFrameShape(QtWidgets.QFrame.StyledPanel)
        self.frame_2.setFrameShadow(QtWidgets.QFrame.Raised)
        self.frame_2.setObjectName("frame_2")
        self.gridLayout_2 = QtWidgets.QGridLayout(self.frame_2)
        self.gridLayout_2.setObjectName("gridLayout_2")
        self.horizontalLayout_14 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_14.setObjectName("horizontalLayout_14")
        self.label_14 = QtWidgets.QLabel(self.frame_2)
        self.label_14.setObjectName("label_14")
        self.horizontalLayout_14.addWidget(self.label_14)
        self.edit_colormap = QtWidgets.QLineEdit(self.frame_2)
        self.edit_colormap.setObjectName("edit_colormap")
        self.horizontalLayout_14.addWidget(self.edit_colormap)
        self.gridLayout_2.addLayout(self.horizontalLayout_14, 6, 0, 1, 3)
        self.label_6 = QtWidgets.QLabel(self.frame_2)
        self.label_6.setObjectName("label_6")
        self.gridLayout_2.addWidget(self.label_6, 0, 0, 1, 1)
        self.radioButton_linear = QtWidgets.QRadioButton(self.frame_2)
        self.radioButton_linear.setChecked(True)
        self.radioButton_linear.setObjectName("radioButton_linear")
        self.gridLayout_2.addWidget(self.radioButton_linear, 1, 0, 1, 1)
        self.radioButton_log = QtWidgets.QRadioButton(self.frame_2)
        self.radioButton_log.setObjectName("radioButton_log")
        self.gridLayout_2.addWidget(self.radioButton_log, 1, 1, 1, 1)
        self.radioButton_loglog = QtWidgets.QRadioButton(self.frame_2)
        self.radioButton_loglog.setObjectName("radioButton_loglog")
        self.gridLayout_2.addWidget(self.radioButton_loglog, 1, 2, 1, 1)
        self.horizontalLayout_7 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_7.setObjectName("horizontalLayout_7")
        self.radioButton_power = QtWidgets.QRadioButton(self.frame_2)
        self.radioButton_power.setObjectName("radioButton_power")
        self.horizontalLayout_7.addWidget(self.radioButton_power)
        spacerItem1 = QtWidgets.QSpacerItem(40, 20, QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Minimum)
        self.horizontalLayout_7.addItem(spacerItem1)
        self.label_7 = QtWidgets.QLabel(self.frame_2)
        self.label_7.setObjectName("label_7")
        self.horizontalLayout_7.addWidget(self.label_7)
        self.edit_Exp = QtWidgets.QLineEdit(self.frame_2)
        self.edit_Exp.setMaximumSize(QtCore.QSize(128, 16777215))
        self.edit_Exp.setObjectName("edit_Exp")
        self.horizontalLayout_7.addWidget(self.edit_Exp)
        self.gridLayout_2.addLayout(self.horizontalLayout_7, 4, 0, 1, 3)
        self.horizontalLayout_9 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_9.setObjectName("horizontalLayout_9")
        self.label_9 = QtWidgets.QLabel(self.frame_2)
        self.label_9.setObjectName("label_9")
        self.horizontalLayout_9.addWidget(self.label_9)
        self.edit_VMax = QtWidgets.QLineEdit(self.frame_2)
        self.edit_VMax.setObjectName("edit_VMax")
        self.horizontalLayout_9.addWidget(self.edit_VMax)
        self.gridLayout_2.addLayout(self.horizontalLayout_9, 5, 0, 1, 3)
        self.verticalLayout_2.addWidget(self.frame_2)
        self.horizontalLayout_8 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_8.setObjectName("horizontalLayout_8")
        self.checkBox_Diffusing = QtWidgets.QCheckBox(self.frame)
        self.checkBox_Diffusing.setObjectName("checkBox_Diffusing")
        self.horizontalLayout_8.addWidget(self.checkBox_Diffusing)
        self.label_8 = QtWidgets.QLabel(self.frame)
        self.label_8.setObjectName("label_8")
        self.horizontalLayout_8.addWidget(self.label_8)
        self.edit_DiffusingSize = QtWidgets.QLineEdit(self.frame)
        self.edit_DiffusingSize.setObjectName("edit_DiffusingSize")
        self.horizontalLayout_8.addWidget(self.edit_DiffusingSize)
        self.verticalLayout_2.addLayout(self.horizontalLayout_8)
        self.line = QtWidgets.QFrame(self.frame)
        self.line.setFrameShape(QtWidgets.QFrame.HLine)
        self.line.setFrameShadow(QtWidgets.QFrame.Sunken)
        self.line.setObjectName("line")
        self.verticalLayout_2.addWidget(self.line)
        self.horizontalLayout_4 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_4.setSizeConstraint(QtWidgets.QLayout.SetDefaultConstraint)
        self.horizontalLayout_4.setObjectName("horizontalLayout_4")
        self.label_3 = QtWidgets.QLabel(self.frame)
        self.label_3.setObjectName("label_3")
        self.horizontalLayout_4.addWidget(self.label_3)
        self.slider_XMin = QtWidgets.QSlider(self.frame)
        self.slider_XMin.setMinimum(0)
        self.slider_XMin.setMaximum(100)
        self.slider_XMin.setSingleStep(1)
        self.slider_XMin.setPageStep(10)
        self.slider_XMin.setProperty("value", 0)
        self.slider_XMin.setOrientation(QtCore.Qt.Horizontal)
        self.slider_XMin.setTickInterval(0)
        self.slider_XMin.setObjectName("slider_XMin")
        self.horizontalLayout_4.addWidget(self.slider_XMin)
        self.edit_XMin = QtWidgets.QLineEdit(self.frame)
        sizePolicy = QtWidgets.QSizePolicy(QtWidgets.QSizePolicy.Expanding, QtWidgets.QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.edit_XMin.sizePolicy().hasHeightForWidth())
        self.edit_XMin.setSizePolicy(sizePolicy)
        self.edit_XMin.setMaximumSize(QtCore.QSize(50, 16777215))
        self.edit_XMin.setObjectName("edit_XMin")
        self.horizontalLayout_4.addWidget(self.edit_XMin)
        self.verticalLayout_2.addLayout(self.horizontalLayout_4)
        self.horizontalLayout_3 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_3.setObjectName("horizontalLayout_3")
        self.label_2 = QtWidgets.QLabel(self.frame)
        self.label_2.setObjectName("label_2")
        self.horizontalLayout_3.addWidget(self.label_2)
        self.slider_XMax = QtWidgets.QSlider(self.frame)
        self.slider_XMax.setMinimum(0)
        self.slider_XMax.setMaximum(100)
        self.slider_XMax.setProperty("value", 100)
        self.slider_XMax.setOrientation(QtCore.Qt.Horizontal)
        self.slider_XMax.setObjectName("slider_XMax")
        self.horizontalLayout_3.addWidget(self.slider_XMax)
        self.edit_XMax = QtWidgets.QLineEdit(self.frame)
        self.edit_XMax.setMaximumSize(QtCore.QSize(50, 16777215))
        self.edit_XMax.setObjectName("edit_XMax")
        self.horizontalLayout_3.addWidget(self.edit_XMax)
        self.verticalLayout_2.addLayout(self.horizontalLayout_3)
        self.checkBox_yrange = QtWidgets.QCheckBox(self.frame)
        self.checkBox_yrange.setCheckable(True)
        self.checkBox_yrange.setChecked(False)
        self.checkBox_yrange.setObjectName("checkBox_yrange")
        self.verticalLayout_2.addWidget(self.checkBox_yrange)
        self.horizontalLayout_6 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_6.setObjectName("horizontalLayout_6")
        self.label_5 = QtWidgets.QLabel(self.frame)
        self.label_5.setObjectName("label_5")
        self.horizontalLayout_6.addWidget(self.label_5)
        self.slider_YMin = QtWidgets.QSlider(self.frame)
        self.slider_YMin.setMinimum(0)
        self.slider_YMin.setMaximum(100)
        self.slider_YMin.setOrientation(QtCore.Qt.Horizontal)
        self.slider_YMin.setObjectName("slider_YMin")
        self.horizontalLayout_6.addWidget(self.slider_YMin)
        self.edit_YMin = QtWidgets.QLineEdit(self.frame)
        self.edit_YMin.setMaximumSize(QtCore.QSize(50, 16777215))
        self.edit_YMin.setReadOnly(False)
        self.edit_YMin.setObjectName("edit_YMin")
        self.horizontalLayout_6.addWidget(self.edit_YMin)
        self.verticalLayout_2.addLayout(self.horizontalLayout_6)
        self.horizontalLayout_5 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_5.setObjectName("horizontalLayout_5")
        self.label_4 = QtWidgets.QLabel(self.frame)
        self.label_4.setObjectName("label_4")
        self.horizontalLayout_5.addWidget(self.label_4)
        self.slider_YMax = QtWidgets.QSlider(self.frame)
        self.slider_YMax.setMinimum(0)
        self.slider_YMax.setMaximum(100)
        self.slider_YMax.setProperty("value", 100)
        self.slider_YMax.setOrientation(QtCore.Qt.Horizontal)
        self.slider_YMax.setObjectName("slider_YMax")
        self.horizontalLayout_5.addWidget(self.slider_YMax)
        self.edit_YMax = QtWidgets.QLineEdit(self.frame)
        self.edit_YMax.setMaximumSize(QtCore.QSize(50, 16777215))
        self.edit_YMax.setReadOnly(False)
        self.edit_YMax.setObjectName("edit_YMax")
        self.horizontalLayout_5.addWidget(self.edit_YMax)
        self.verticalLayout_2.addLayout(self.horizontalLayout_5)
        self.horizontalLayout_2 = QtWidgets.QHBoxLayout()
        self.horizontalLayout_2.setObjectName("horizontalLayout_2")
        self.label = QtWidgets.QLabel(self.frame)
        self.label.setObjectName("label")
        self.horizontalLayout_2.addWidget(self.label)
        self.edit_XBins = QtWidgets.QLineEdit(self.frame)
        self.edit_XBins.setObjectName("edit_XBins")
        self.horizontalLayout_2.addWidget(self.edit_XBins)
        self.label_15 = QtWidgets.QLabel(self.frame)
        self.label_15.setObjectName("label_15")
        self.horizontalLayout_2.addWidget(self.label_15)
        self.edit_YBins = QtWidgets.QLineEdit(self.frame)
        self.edit_YBins.setObjectName("edit_YBins")
        self.horizontalLayout_2.addWidget(self.edit_YBins)
        self.verticalLayout_2.addLayout(self.horizontalLayout_2)
        self.buttonReplot = QtWidgets.QPushButton(self.frame)
        self.buttonReplot.setObjectName("buttonReplot")
        self.verticalLayout_2.addWidget(self.buttonReplot)
        self.buttonUndo = QtWidgets.QPushButton(self.frame)
        self.buttonUndo.setObjectName("buttonUndo")
        self.verticalLayout_2.addWidget(self.buttonUndo)
        self.horizontalLayout.addWidget(self.frame)
        MainWindow.setCentralWidget(self.centralwidget)
        self.menubar = QtWidgets.QMenuBar(MainWindow)
        self.menubar.setGeometry(QtCore.QRect(0, 0, 876, 20))
        self.menubar.setObjectName("menubar")
        self.menuFile = QtWidgets.QMenu(self.menubar)
        self.menuFile.setObjectName("menuFile")
        MainWindow.setMenuBar(self.menubar)
        self.statusbar = QtWidgets.QStatusBar(MainWindow)
        self.statusbar.setObjectName("statusbar")
        MainWindow.setStatusBar(self.statusbar)
        self.toolBar = QtWidgets.QToolBar(MainWindow)
        self.toolBar.setObjectName("toolBar")
        MainWindow.addToolBar(QtCore.Qt.TopToolBarArea, self.toolBar)
        self.actionOpen = QtWidgets.QAction(MainWindow)
        self.actionOpen.setObjectName("actionOpen")
        self.actionQuit = QtWidgets.QAction(MainWindow)
        self.actionQuit.setObjectName("actionQuit")
        self.actionMeasure = QtWidgets.QAction(MainWindow)
        self.actionMeasure.setCheckable(True)
        self.actionMeasure.setObjectName("actionMeasure")
        self.actionRange = QtWidgets.QAction(MainWindow)
        self.actionRange.setCheckable(True)
        self.actionRange.setObjectName("actionRange")
        self.actionCount = QtWidgets.QAction(MainWindow)
        self.actionCount.setCheckable(True)
        self.actionCount.setObjectName("actionCount")
        self.menuFile.addAction(self.actionOpen)
        self.menuFile.addSeparator()
        self.menuFile.addAction(self.actionQuit)
        self.menubar.addAction(self.menuFile.menuAction())
        self.toolBar.addAction(self.actionMeasure)
        self.toolBar.addAction(self.actionRange)
        self.toolBar.addAction(self.actionCount)

        self.retranslateUi(MainWindow)
        QtCore.QMetaObject.connectSlotsByName(MainWindow)

    def retranslateUi(self, MainWindow):
        _translate = QtCore.QCoreApplication.translate
        MainWindow.setWindowTitle(_translate("MainWindow", "plot_PD"))
        self.label_13.setText(_translate("MainWindow", "eigenvolume threshold:"))
        self.lineEdit_optimal_volume_stable_volume.setText(_translate("MainWindow", "0.001"))
        self.radioButton_eigenvolume_tightened.setText(_translate("MainWindow", "tightened"))
        self.radioButton_eigenvolume_tightened_sub.setText(_translate("MainWindow", "tightened sub"))
        self.label_12.setText(_translate("MainWindow", "Retry: "))
        self.lineEdit_optimal_volume_retry.setText(_translate("MainWindow", "4"))
        self.label_11.setText(_translate("MainWindow", "cutoff-radius:"))
        self.lineEdit_optimal_volume_cutoff_radius.setText(_translate("MainWindow", "0.01"))
        self.radioButton_eigenvolume_owned.setText(_translate("MainWindow", "owned"))
        self.radioButton_eigenvolume_none.setText(_translate("MainWindow", "None"))
        self.checkBox_optimal_volume_query_children.setText(_translate("MainWindow", "Query children (slower)"))
        self.checkBox_optimal_volume_on.setText(_translate("MainWindow", "Query volume optimal cycle"))
        self.checkBox_optimal_volume_show_children.setText(_translate("MainWindow", "Show children"))
        self.checkBox_optimal_volume_show_volume.setText(_translate("MainWindow", "Show volume"))
        self.label_16.setText(_translate("MainWindow", "Eigenvolume:"))
        self.button_show_descendants.setText(_translate("MainWindow", "Show descendants volume"))
        self.checkBox_query_volumes_in_rectangle.setText(_translate("MainWindow", "Query volumes in rectangle"))
        self.checkBox_show_descendants.setText(_translate("MainWindow", "Show descendants"))
        self.checkBox_show_ancestors.setText(_translate("MainWindow", "Show ancestors"))
        self.checkBox_query_phtree.setText(_translate("MainWindow", "Query tree"))
        self.label_10.setText(_translate("MainWindow", "Plotting style:"))
        self.radioButton_normalplot.setText(_translate("MainWindow", "&normal"))
        self.radioButton_contourplot.setText(_translate("MainWindow", "&contour"))
        self.label_14.setText(_translate("MainWindow", "colormap: "))
        self.label_6.setText(_translate("MainWindow", "Norm:"))
        self.radioButton_linear.setText(_translate("MainWindow", "&linear"))
        self.radioButton_log.setText(_translate("MainWindow", "lo&g"))
        self.radioButton_loglog.setText(_translate("MainWindow", "lo&glog"))
        self.radioButton_power.setText(_translate("MainWindow", "&power"))
        self.label_7.setText(_translate("MainWindow", "exp:"))
        self.edit_Exp.setText(_translate("MainWindow", "1"))
        self.label_9.setText(_translate("MainWindow", "max:"))
        self.checkBox_Diffusing.setText(_translate("MainWindow", "diffusing pairs"))
        self.label_8.setText(_translate("MainWindow", "size:"))
        self.label_3.setText(_translate("MainWindow", "X min:"))
        self.edit_XMin.setText(_translate("MainWindow", "0"))
        self.label_2.setText(_translate("MainWindow", "X max:"))
        self.edit_XMax.setText(_translate("MainWindow", "0.1"))
        self.checkBox_yrange.setText(_translate("MainWindow", "xrange == yrange"))
        self.label_5.setText(_translate("MainWindow", "Y min:"))
        self.edit_YMin.setText(_translate("MainWindow", "0"))
        self.label_4.setText(_translate("MainWindow", "Y max:"))
        self.edit_YMax.setText(_translate("MainWindow", "0.1"))
        self.label.setText(_translate("MainWindow", "XBins:"))
        self.edit_XBins.setText(_translate("MainWindow", "128"))
        self.label_15.setText(_translate("MainWindow", "YBins: "))
        self.buttonReplot.setText(_translate("MainWindow", "Replot"))
        self.buttonUndo.setText(_translate("MainWindow", "Undo"))
        self.menuFile.setTitle(_translate("MainWindow", "F&ile"))
        self.toolBar.setWindowTitle(_translate("MainWindow", "toolBar"))
        self.actionOpen.setText(_translate("MainWindow", "&Open"))
        self.actionQuit.setText(_translate("MainWindow", "&Quit"))
        self.actionQuit.setShortcut(_translate("MainWindow", "Ctrl+Q"))
        self.actionMeasure.setText(_translate("MainWindow", "Measure"))
        self.actionMeasure.setToolTip(_translate("MainWindow", "Measure distance"))
        self.actionRange.setText(_translate("MainWindow", "Range"))
        self.actionRange.setToolTip(_translate("MainWindow", "Range specification"))
        self.actionCount.setText(_translate("MainWindow", "Count"))
