# /* **************************************************************************
#  *                                                                          *
#  *     (C) Copyright Paul Mensonides 2002.
#  *     Distributed under the Boost Software License, Version 1.0. (See
#  *     accompanying file LICENSE_1_0.txt or copy at
#  *     http://www.boost.org/LICENSE_1_0.txt)
#  *                                                                          *
#  ************************************************************************** */
#
# /* See http://www.boost.org for most recent version. */
#
# if defined(MSGPACK_PP_ITERATION_LIMITS)
#    if !defined(MSGPACK_PP_FILENAME_2)
#        error MSGPACK_PP_ERROR:  depth #2 filename is not defined
#    endif
#    define MSGPACK_PP_VALUE MSGPACK_PP_TUPLE_ELEM(2, 0, MSGPACK_PP_ITERATION_LIMITS)
#    include <msgpack/preprocessor/iteration/detail/bounds/lower2.hpp>
#    define MSGPACK_PP_VALUE MSGPACK_PP_TUPLE_ELEM(2, 1, MSGPACK_PP_ITERATION_LIMITS)
#    include <msgpack/preprocessor/iteration/detail/bounds/upper2.hpp>
#    define MSGPACK_PP_ITERATION_FLAGS_2() 0
#    undef MSGPACK_PP_ITERATION_LIMITS
# elif defined(MSGPACK_PP_ITERATION_PARAMS_2)
#    define MSGPACK_PP_VALUE MSGPACK_PP_ARRAY_ELEM(0, MSGPACK_PP_ITERATION_PARAMS_2)
#    include <msgpack/preprocessor/iteration/detail/bounds/lower2.hpp>
#    define MSGPACK_PP_VALUE MSGPACK_PP_ARRAY_ELEM(1, MSGPACK_PP_ITERATION_PARAMS_2)
#    include <msgpack/preprocessor/iteration/detail/bounds/upper2.hpp>
#    define MSGPACK_PP_FILENAME_2 MSGPACK_PP_ARRAY_ELEM(2, MSGPACK_PP_ITERATION_PARAMS_2)
#    if MSGPACK_PP_ARRAY_SIZE(MSGPACK_PP_ITERATION_PARAMS_2) >= 4
#        define MSGPACK_PP_ITERATION_FLAGS_2() MSGPACK_PP_ARRAY_ELEM(3, MSGPACK_PP_ITERATION_PARAMS_2)
#    else
#        define MSGPACK_PP_ITERATION_FLAGS_2() 0
#    endif
# else
#    error MSGPACK_PP_ERROR:  depth #2 iteration boundaries or filename not defined
# endif
#
# undef MSGPACK_PP_ITERATION_DEPTH
# define MSGPACK_PP_ITERATION_DEPTH() 2
#
# if (MSGPACK_PP_ITERATION_START_2) > (MSGPACK_PP_ITERATION_FINISH_2)
#    include <msgpack/preprocessor/iteration/detail/iter/reverse2.hpp>
# else
#    if MSGPACK_PP_ITERATION_START_2 <= 0 && MSGPACK_PP_ITERATION_FINISH_2 >= 0
#        define MSGPACK_PP_ITERATION_2 0
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 1 && MSGPACK_PP_ITERATION_FINISH_2 >= 1
#        define MSGPACK_PP_ITERATION_2 1
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 2 && MSGPACK_PP_ITERATION_FINISH_2 >= 2
#        define MSGPACK_PP_ITERATION_2 2
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 3 && MSGPACK_PP_ITERATION_FINISH_2 >= 3
#        define MSGPACK_PP_ITERATION_2 3
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 4 && MSGPACK_PP_ITERATION_FINISH_2 >= 4
#        define MSGPACK_PP_ITERATION_2 4
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 5 && MSGPACK_PP_ITERATION_FINISH_2 >= 5
#        define MSGPACK_PP_ITERATION_2 5
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 6 && MSGPACK_PP_ITERATION_FINISH_2 >= 6
#        define MSGPACK_PP_ITERATION_2 6
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 7 && MSGPACK_PP_ITERATION_FINISH_2 >= 7
#        define MSGPACK_PP_ITERATION_2 7
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 8 && MSGPACK_PP_ITERATION_FINISH_2 >= 8
#        define MSGPACK_PP_ITERATION_2 8
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 9 && MSGPACK_PP_ITERATION_FINISH_2 >= 9
#        define MSGPACK_PP_ITERATION_2 9
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 10 && MSGPACK_PP_ITERATION_FINISH_2 >= 10
#        define MSGPACK_PP_ITERATION_2 10
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 11 && MSGPACK_PP_ITERATION_FINISH_2 >= 11
#        define MSGPACK_PP_ITERATION_2 11
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 12 && MSGPACK_PP_ITERATION_FINISH_2 >= 12
#        define MSGPACK_PP_ITERATION_2 12
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 13 && MSGPACK_PP_ITERATION_FINISH_2 >= 13
#        define MSGPACK_PP_ITERATION_2 13
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 14 && MSGPACK_PP_ITERATION_FINISH_2 >= 14
#        define MSGPACK_PP_ITERATION_2 14
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 15 && MSGPACK_PP_ITERATION_FINISH_2 >= 15
#        define MSGPACK_PP_ITERATION_2 15
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 16 && MSGPACK_PP_ITERATION_FINISH_2 >= 16
#        define MSGPACK_PP_ITERATION_2 16
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 17 && MSGPACK_PP_ITERATION_FINISH_2 >= 17
#        define MSGPACK_PP_ITERATION_2 17
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 18 && MSGPACK_PP_ITERATION_FINISH_2 >= 18
#        define MSGPACK_PP_ITERATION_2 18
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 19 && MSGPACK_PP_ITERATION_FINISH_2 >= 19
#        define MSGPACK_PP_ITERATION_2 19
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 20 && MSGPACK_PP_ITERATION_FINISH_2 >= 20
#        define MSGPACK_PP_ITERATION_2 20
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 21 && MSGPACK_PP_ITERATION_FINISH_2 >= 21
#        define MSGPACK_PP_ITERATION_2 21
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 22 && MSGPACK_PP_ITERATION_FINISH_2 >= 22
#        define MSGPACK_PP_ITERATION_2 22
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 23 && MSGPACK_PP_ITERATION_FINISH_2 >= 23
#        define MSGPACK_PP_ITERATION_2 23
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 24 && MSGPACK_PP_ITERATION_FINISH_2 >= 24
#        define MSGPACK_PP_ITERATION_2 24
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 25 && MSGPACK_PP_ITERATION_FINISH_2 >= 25
#        define MSGPACK_PP_ITERATION_2 25
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 26 && MSGPACK_PP_ITERATION_FINISH_2 >= 26
#        define MSGPACK_PP_ITERATION_2 26
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 27 && MSGPACK_PP_ITERATION_FINISH_2 >= 27
#        define MSGPACK_PP_ITERATION_2 27
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 28 && MSGPACK_PP_ITERATION_FINISH_2 >= 28
#        define MSGPACK_PP_ITERATION_2 28
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 29 && MSGPACK_PP_ITERATION_FINISH_2 >= 29
#        define MSGPACK_PP_ITERATION_2 29
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 30 && MSGPACK_PP_ITERATION_FINISH_2 >= 30
#        define MSGPACK_PP_ITERATION_2 30
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 31 && MSGPACK_PP_ITERATION_FINISH_2 >= 31
#        define MSGPACK_PP_ITERATION_2 31
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 32 && MSGPACK_PP_ITERATION_FINISH_2 >= 32
#        define MSGPACK_PP_ITERATION_2 32
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 33 && MSGPACK_PP_ITERATION_FINISH_2 >= 33
#        define MSGPACK_PP_ITERATION_2 33
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 34 && MSGPACK_PP_ITERATION_FINISH_2 >= 34
#        define MSGPACK_PP_ITERATION_2 34
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 35 && MSGPACK_PP_ITERATION_FINISH_2 >= 35
#        define MSGPACK_PP_ITERATION_2 35
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 36 && MSGPACK_PP_ITERATION_FINISH_2 >= 36
#        define MSGPACK_PP_ITERATION_2 36
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 37 && MSGPACK_PP_ITERATION_FINISH_2 >= 37
#        define MSGPACK_PP_ITERATION_2 37
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 38 && MSGPACK_PP_ITERATION_FINISH_2 >= 38
#        define MSGPACK_PP_ITERATION_2 38
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 39 && MSGPACK_PP_ITERATION_FINISH_2 >= 39
#        define MSGPACK_PP_ITERATION_2 39
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 40 && MSGPACK_PP_ITERATION_FINISH_2 >= 40
#        define MSGPACK_PP_ITERATION_2 40
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 41 && MSGPACK_PP_ITERATION_FINISH_2 >= 41
#        define MSGPACK_PP_ITERATION_2 41
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 42 && MSGPACK_PP_ITERATION_FINISH_2 >= 42
#        define MSGPACK_PP_ITERATION_2 42
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 43 && MSGPACK_PP_ITERATION_FINISH_2 >= 43
#        define MSGPACK_PP_ITERATION_2 43
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 44 && MSGPACK_PP_ITERATION_FINISH_2 >= 44
#        define MSGPACK_PP_ITERATION_2 44
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 45 && MSGPACK_PP_ITERATION_FINISH_2 >= 45
#        define MSGPACK_PP_ITERATION_2 45
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 46 && MSGPACK_PP_ITERATION_FINISH_2 >= 46
#        define MSGPACK_PP_ITERATION_2 46
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 47 && MSGPACK_PP_ITERATION_FINISH_2 >= 47
#        define MSGPACK_PP_ITERATION_2 47
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 48 && MSGPACK_PP_ITERATION_FINISH_2 >= 48
#        define MSGPACK_PP_ITERATION_2 48
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 49 && MSGPACK_PP_ITERATION_FINISH_2 >= 49
#        define MSGPACK_PP_ITERATION_2 49
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 50 && MSGPACK_PP_ITERATION_FINISH_2 >= 50
#        define MSGPACK_PP_ITERATION_2 50
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 51 && MSGPACK_PP_ITERATION_FINISH_2 >= 51
#        define MSGPACK_PP_ITERATION_2 51
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 52 && MSGPACK_PP_ITERATION_FINISH_2 >= 52
#        define MSGPACK_PP_ITERATION_2 52
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 53 && MSGPACK_PP_ITERATION_FINISH_2 >= 53
#        define MSGPACK_PP_ITERATION_2 53
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 54 && MSGPACK_PP_ITERATION_FINISH_2 >= 54
#        define MSGPACK_PP_ITERATION_2 54
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 55 && MSGPACK_PP_ITERATION_FINISH_2 >= 55
#        define MSGPACK_PP_ITERATION_2 55
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 56 && MSGPACK_PP_ITERATION_FINISH_2 >= 56
#        define MSGPACK_PP_ITERATION_2 56
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 57 && MSGPACK_PP_ITERATION_FINISH_2 >= 57
#        define MSGPACK_PP_ITERATION_2 57
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 58 && MSGPACK_PP_ITERATION_FINISH_2 >= 58
#        define MSGPACK_PP_ITERATION_2 58
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 59 && MSGPACK_PP_ITERATION_FINISH_2 >= 59
#        define MSGPACK_PP_ITERATION_2 59
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 60 && MSGPACK_PP_ITERATION_FINISH_2 >= 60
#        define MSGPACK_PP_ITERATION_2 60
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 61 && MSGPACK_PP_ITERATION_FINISH_2 >= 61
#        define MSGPACK_PP_ITERATION_2 61
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 62 && MSGPACK_PP_ITERATION_FINISH_2 >= 62
#        define MSGPACK_PP_ITERATION_2 62
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 63 && MSGPACK_PP_ITERATION_FINISH_2 >= 63
#        define MSGPACK_PP_ITERATION_2 63
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 64 && MSGPACK_PP_ITERATION_FINISH_2 >= 64
#        define MSGPACK_PP_ITERATION_2 64
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 65 && MSGPACK_PP_ITERATION_FINISH_2 >= 65
#        define MSGPACK_PP_ITERATION_2 65
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 66 && MSGPACK_PP_ITERATION_FINISH_2 >= 66
#        define MSGPACK_PP_ITERATION_2 66
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 67 && MSGPACK_PP_ITERATION_FINISH_2 >= 67
#        define MSGPACK_PP_ITERATION_2 67
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 68 && MSGPACK_PP_ITERATION_FINISH_2 >= 68
#        define MSGPACK_PP_ITERATION_2 68
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 69 && MSGPACK_PP_ITERATION_FINISH_2 >= 69
#        define MSGPACK_PP_ITERATION_2 69
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 70 && MSGPACK_PP_ITERATION_FINISH_2 >= 70
#        define MSGPACK_PP_ITERATION_2 70
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 71 && MSGPACK_PP_ITERATION_FINISH_2 >= 71
#        define MSGPACK_PP_ITERATION_2 71
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 72 && MSGPACK_PP_ITERATION_FINISH_2 >= 72
#        define MSGPACK_PP_ITERATION_2 72
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 73 && MSGPACK_PP_ITERATION_FINISH_2 >= 73
#        define MSGPACK_PP_ITERATION_2 73
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 74 && MSGPACK_PP_ITERATION_FINISH_2 >= 74
#        define MSGPACK_PP_ITERATION_2 74
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 75 && MSGPACK_PP_ITERATION_FINISH_2 >= 75
#        define MSGPACK_PP_ITERATION_2 75
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 76 && MSGPACK_PP_ITERATION_FINISH_2 >= 76
#        define MSGPACK_PP_ITERATION_2 76
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 77 && MSGPACK_PP_ITERATION_FINISH_2 >= 77
#        define MSGPACK_PP_ITERATION_2 77
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 78 && MSGPACK_PP_ITERATION_FINISH_2 >= 78
#        define MSGPACK_PP_ITERATION_2 78
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 79 && MSGPACK_PP_ITERATION_FINISH_2 >= 79
#        define MSGPACK_PP_ITERATION_2 79
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 80 && MSGPACK_PP_ITERATION_FINISH_2 >= 80
#        define MSGPACK_PP_ITERATION_2 80
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 81 && MSGPACK_PP_ITERATION_FINISH_2 >= 81
#        define MSGPACK_PP_ITERATION_2 81
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 82 && MSGPACK_PP_ITERATION_FINISH_2 >= 82
#        define MSGPACK_PP_ITERATION_2 82
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 83 && MSGPACK_PP_ITERATION_FINISH_2 >= 83
#        define MSGPACK_PP_ITERATION_2 83
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 84 && MSGPACK_PP_ITERATION_FINISH_2 >= 84
#        define MSGPACK_PP_ITERATION_2 84
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 85 && MSGPACK_PP_ITERATION_FINISH_2 >= 85
#        define MSGPACK_PP_ITERATION_2 85
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 86 && MSGPACK_PP_ITERATION_FINISH_2 >= 86
#        define MSGPACK_PP_ITERATION_2 86
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 87 && MSGPACK_PP_ITERATION_FINISH_2 >= 87
#        define MSGPACK_PP_ITERATION_2 87
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 88 && MSGPACK_PP_ITERATION_FINISH_2 >= 88
#        define MSGPACK_PP_ITERATION_2 88
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 89 && MSGPACK_PP_ITERATION_FINISH_2 >= 89
#        define MSGPACK_PP_ITERATION_2 89
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 90 && MSGPACK_PP_ITERATION_FINISH_2 >= 90
#        define MSGPACK_PP_ITERATION_2 90
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 91 && MSGPACK_PP_ITERATION_FINISH_2 >= 91
#        define MSGPACK_PP_ITERATION_2 91
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 92 && MSGPACK_PP_ITERATION_FINISH_2 >= 92
#        define MSGPACK_PP_ITERATION_2 92
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 93 && MSGPACK_PP_ITERATION_FINISH_2 >= 93
#        define MSGPACK_PP_ITERATION_2 93
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 94 && MSGPACK_PP_ITERATION_FINISH_2 >= 94
#        define MSGPACK_PP_ITERATION_2 94
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 95 && MSGPACK_PP_ITERATION_FINISH_2 >= 95
#        define MSGPACK_PP_ITERATION_2 95
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 96 && MSGPACK_PP_ITERATION_FINISH_2 >= 96
#        define MSGPACK_PP_ITERATION_2 96
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 97 && MSGPACK_PP_ITERATION_FINISH_2 >= 97
#        define MSGPACK_PP_ITERATION_2 97
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 98 && MSGPACK_PP_ITERATION_FINISH_2 >= 98
#        define MSGPACK_PP_ITERATION_2 98
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 99 && MSGPACK_PP_ITERATION_FINISH_2 >= 99
#        define MSGPACK_PP_ITERATION_2 99
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 100 && MSGPACK_PP_ITERATION_FINISH_2 >= 100
#        define MSGPACK_PP_ITERATION_2 100
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 101 && MSGPACK_PP_ITERATION_FINISH_2 >= 101
#        define MSGPACK_PP_ITERATION_2 101
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 102 && MSGPACK_PP_ITERATION_FINISH_2 >= 102
#        define MSGPACK_PP_ITERATION_2 102
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 103 && MSGPACK_PP_ITERATION_FINISH_2 >= 103
#        define MSGPACK_PP_ITERATION_2 103
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 104 && MSGPACK_PP_ITERATION_FINISH_2 >= 104
#        define MSGPACK_PP_ITERATION_2 104
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 105 && MSGPACK_PP_ITERATION_FINISH_2 >= 105
#        define MSGPACK_PP_ITERATION_2 105
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 106 && MSGPACK_PP_ITERATION_FINISH_2 >= 106
#        define MSGPACK_PP_ITERATION_2 106
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 107 && MSGPACK_PP_ITERATION_FINISH_2 >= 107
#        define MSGPACK_PP_ITERATION_2 107
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 108 && MSGPACK_PP_ITERATION_FINISH_2 >= 108
#        define MSGPACK_PP_ITERATION_2 108
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 109 && MSGPACK_PP_ITERATION_FINISH_2 >= 109
#        define MSGPACK_PP_ITERATION_2 109
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 110 && MSGPACK_PP_ITERATION_FINISH_2 >= 110
#        define MSGPACK_PP_ITERATION_2 110
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 111 && MSGPACK_PP_ITERATION_FINISH_2 >= 111
#        define MSGPACK_PP_ITERATION_2 111
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 112 && MSGPACK_PP_ITERATION_FINISH_2 >= 112
#        define MSGPACK_PP_ITERATION_2 112
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 113 && MSGPACK_PP_ITERATION_FINISH_2 >= 113
#        define MSGPACK_PP_ITERATION_2 113
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 114 && MSGPACK_PP_ITERATION_FINISH_2 >= 114
#        define MSGPACK_PP_ITERATION_2 114
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 115 && MSGPACK_PP_ITERATION_FINISH_2 >= 115
#        define MSGPACK_PP_ITERATION_2 115
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 116 && MSGPACK_PP_ITERATION_FINISH_2 >= 116
#        define MSGPACK_PP_ITERATION_2 116
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 117 && MSGPACK_PP_ITERATION_FINISH_2 >= 117
#        define MSGPACK_PP_ITERATION_2 117
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 118 && MSGPACK_PP_ITERATION_FINISH_2 >= 118
#        define MSGPACK_PP_ITERATION_2 118
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 119 && MSGPACK_PP_ITERATION_FINISH_2 >= 119
#        define MSGPACK_PP_ITERATION_2 119
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 120 && MSGPACK_PP_ITERATION_FINISH_2 >= 120
#        define MSGPACK_PP_ITERATION_2 120
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 121 && MSGPACK_PP_ITERATION_FINISH_2 >= 121
#        define MSGPACK_PP_ITERATION_2 121
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 122 && MSGPACK_PP_ITERATION_FINISH_2 >= 122
#        define MSGPACK_PP_ITERATION_2 122
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 123 && MSGPACK_PP_ITERATION_FINISH_2 >= 123
#        define MSGPACK_PP_ITERATION_2 123
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 124 && MSGPACK_PP_ITERATION_FINISH_2 >= 124
#        define MSGPACK_PP_ITERATION_2 124
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 125 && MSGPACK_PP_ITERATION_FINISH_2 >= 125
#        define MSGPACK_PP_ITERATION_2 125
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 126 && MSGPACK_PP_ITERATION_FINISH_2 >= 126
#        define MSGPACK_PP_ITERATION_2 126
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 127 && MSGPACK_PP_ITERATION_FINISH_2 >= 127
#        define MSGPACK_PP_ITERATION_2 127
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 128 && MSGPACK_PP_ITERATION_FINISH_2 >= 128
#        define MSGPACK_PP_ITERATION_2 128
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 129 && MSGPACK_PP_ITERATION_FINISH_2 >= 129
#        define MSGPACK_PP_ITERATION_2 129
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 130 && MSGPACK_PP_ITERATION_FINISH_2 >= 130
#        define MSGPACK_PP_ITERATION_2 130
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 131 && MSGPACK_PP_ITERATION_FINISH_2 >= 131
#        define MSGPACK_PP_ITERATION_2 131
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 132 && MSGPACK_PP_ITERATION_FINISH_2 >= 132
#        define MSGPACK_PP_ITERATION_2 132
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 133 && MSGPACK_PP_ITERATION_FINISH_2 >= 133
#        define MSGPACK_PP_ITERATION_2 133
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 134 && MSGPACK_PP_ITERATION_FINISH_2 >= 134
#        define MSGPACK_PP_ITERATION_2 134
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 135 && MSGPACK_PP_ITERATION_FINISH_2 >= 135
#        define MSGPACK_PP_ITERATION_2 135
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 136 && MSGPACK_PP_ITERATION_FINISH_2 >= 136
#        define MSGPACK_PP_ITERATION_2 136
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 137 && MSGPACK_PP_ITERATION_FINISH_2 >= 137
#        define MSGPACK_PP_ITERATION_2 137
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 138 && MSGPACK_PP_ITERATION_FINISH_2 >= 138
#        define MSGPACK_PP_ITERATION_2 138
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 139 && MSGPACK_PP_ITERATION_FINISH_2 >= 139
#        define MSGPACK_PP_ITERATION_2 139
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 140 && MSGPACK_PP_ITERATION_FINISH_2 >= 140
#        define MSGPACK_PP_ITERATION_2 140
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 141 && MSGPACK_PP_ITERATION_FINISH_2 >= 141
#        define MSGPACK_PP_ITERATION_2 141
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 142 && MSGPACK_PP_ITERATION_FINISH_2 >= 142
#        define MSGPACK_PP_ITERATION_2 142
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 143 && MSGPACK_PP_ITERATION_FINISH_2 >= 143
#        define MSGPACK_PP_ITERATION_2 143
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 144 && MSGPACK_PP_ITERATION_FINISH_2 >= 144
#        define MSGPACK_PP_ITERATION_2 144
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 145 && MSGPACK_PP_ITERATION_FINISH_2 >= 145
#        define MSGPACK_PP_ITERATION_2 145
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 146 && MSGPACK_PP_ITERATION_FINISH_2 >= 146
#        define MSGPACK_PP_ITERATION_2 146
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 147 && MSGPACK_PP_ITERATION_FINISH_2 >= 147
#        define MSGPACK_PP_ITERATION_2 147
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 148 && MSGPACK_PP_ITERATION_FINISH_2 >= 148
#        define MSGPACK_PP_ITERATION_2 148
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 149 && MSGPACK_PP_ITERATION_FINISH_2 >= 149
#        define MSGPACK_PP_ITERATION_2 149
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 150 && MSGPACK_PP_ITERATION_FINISH_2 >= 150
#        define MSGPACK_PP_ITERATION_2 150
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 151 && MSGPACK_PP_ITERATION_FINISH_2 >= 151
#        define MSGPACK_PP_ITERATION_2 151
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 152 && MSGPACK_PP_ITERATION_FINISH_2 >= 152
#        define MSGPACK_PP_ITERATION_2 152
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 153 && MSGPACK_PP_ITERATION_FINISH_2 >= 153
#        define MSGPACK_PP_ITERATION_2 153
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 154 && MSGPACK_PP_ITERATION_FINISH_2 >= 154
#        define MSGPACK_PP_ITERATION_2 154
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 155 && MSGPACK_PP_ITERATION_FINISH_2 >= 155
#        define MSGPACK_PP_ITERATION_2 155
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 156 && MSGPACK_PP_ITERATION_FINISH_2 >= 156
#        define MSGPACK_PP_ITERATION_2 156
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 157 && MSGPACK_PP_ITERATION_FINISH_2 >= 157
#        define MSGPACK_PP_ITERATION_2 157
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 158 && MSGPACK_PP_ITERATION_FINISH_2 >= 158
#        define MSGPACK_PP_ITERATION_2 158
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 159 && MSGPACK_PP_ITERATION_FINISH_2 >= 159
#        define MSGPACK_PP_ITERATION_2 159
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 160 && MSGPACK_PP_ITERATION_FINISH_2 >= 160
#        define MSGPACK_PP_ITERATION_2 160
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 161 && MSGPACK_PP_ITERATION_FINISH_2 >= 161
#        define MSGPACK_PP_ITERATION_2 161
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 162 && MSGPACK_PP_ITERATION_FINISH_2 >= 162
#        define MSGPACK_PP_ITERATION_2 162
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 163 && MSGPACK_PP_ITERATION_FINISH_2 >= 163
#        define MSGPACK_PP_ITERATION_2 163
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 164 && MSGPACK_PP_ITERATION_FINISH_2 >= 164
#        define MSGPACK_PP_ITERATION_2 164
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 165 && MSGPACK_PP_ITERATION_FINISH_2 >= 165
#        define MSGPACK_PP_ITERATION_2 165
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 166 && MSGPACK_PP_ITERATION_FINISH_2 >= 166
#        define MSGPACK_PP_ITERATION_2 166
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 167 && MSGPACK_PP_ITERATION_FINISH_2 >= 167
#        define MSGPACK_PP_ITERATION_2 167
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 168 && MSGPACK_PP_ITERATION_FINISH_2 >= 168
#        define MSGPACK_PP_ITERATION_2 168
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 169 && MSGPACK_PP_ITERATION_FINISH_2 >= 169
#        define MSGPACK_PP_ITERATION_2 169
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 170 && MSGPACK_PP_ITERATION_FINISH_2 >= 170
#        define MSGPACK_PP_ITERATION_2 170
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 171 && MSGPACK_PP_ITERATION_FINISH_2 >= 171
#        define MSGPACK_PP_ITERATION_2 171
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 172 && MSGPACK_PP_ITERATION_FINISH_2 >= 172
#        define MSGPACK_PP_ITERATION_2 172
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 173 && MSGPACK_PP_ITERATION_FINISH_2 >= 173
#        define MSGPACK_PP_ITERATION_2 173
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 174 && MSGPACK_PP_ITERATION_FINISH_2 >= 174
#        define MSGPACK_PP_ITERATION_2 174
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 175 && MSGPACK_PP_ITERATION_FINISH_2 >= 175
#        define MSGPACK_PP_ITERATION_2 175
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 176 && MSGPACK_PP_ITERATION_FINISH_2 >= 176
#        define MSGPACK_PP_ITERATION_2 176
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 177 && MSGPACK_PP_ITERATION_FINISH_2 >= 177
#        define MSGPACK_PP_ITERATION_2 177
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 178 && MSGPACK_PP_ITERATION_FINISH_2 >= 178
#        define MSGPACK_PP_ITERATION_2 178
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 179 && MSGPACK_PP_ITERATION_FINISH_2 >= 179
#        define MSGPACK_PP_ITERATION_2 179
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 180 && MSGPACK_PP_ITERATION_FINISH_2 >= 180
#        define MSGPACK_PP_ITERATION_2 180
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 181 && MSGPACK_PP_ITERATION_FINISH_2 >= 181
#        define MSGPACK_PP_ITERATION_2 181
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 182 && MSGPACK_PP_ITERATION_FINISH_2 >= 182
#        define MSGPACK_PP_ITERATION_2 182
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 183 && MSGPACK_PP_ITERATION_FINISH_2 >= 183
#        define MSGPACK_PP_ITERATION_2 183
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 184 && MSGPACK_PP_ITERATION_FINISH_2 >= 184
#        define MSGPACK_PP_ITERATION_2 184
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 185 && MSGPACK_PP_ITERATION_FINISH_2 >= 185
#        define MSGPACK_PP_ITERATION_2 185
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 186 && MSGPACK_PP_ITERATION_FINISH_2 >= 186
#        define MSGPACK_PP_ITERATION_2 186
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 187 && MSGPACK_PP_ITERATION_FINISH_2 >= 187
#        define MSGPACK_PP_ITERATION_2 187
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 188 && MSGPACK_PP_ITERATION_FINISH_2 >= 188
#        define MSGPACK_PP_ITERATION_2 188
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 189 && MSGPACK_PP_ITERATION_FINISH_2 >= 189
#        define MSGPACK_PP_ITERATION_2 189
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 190 && MSGPACK_PP_ITERATION_FINISH_2 >= 190
#        define MSGPACK_PP_ITERATION_2 190
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 191 && MSGPACK_PP_ITERATION_FINISH_2 >= 191
#        define MSGPACK_PP_ITERATION_2 191
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 192 && MSGPACK_PP_ITERATION_FINISH_2 >= 192
#        define MSGPACK_PP_ITERATION_2 192
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 193 && MSGPACK_PP_ITERATION_FINISH_2 >= 193
#        define MSGPACK_PP_ITERATION_2 193
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 194 && MSGPACK_PP_ITERATION_FINISH_2 >= 194
#        define MSGPACK_PP_ITERATION_2 194
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 195 && MSGPACK_PP_ITERATION_FINISH_2 >= 195
#        define MSGPACK_PP_ITERATION_2 195
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 196 && MSGPACK_PP_ITERATION_FINISH_2 >= 196
#        define MSGPACK_PP_ITERATION_2 196
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 197 && MSGPACK_PP_ITERATION_FINISH_2 >= 197
#        define MSGPACK_PP_ITERATION_2 197
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 198 && MSGPACK_PP_ITERATION_FINISH_2 >= 198
#        define MSGPACK_PP_ITERATION_2 198
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 199 && MSGPACK_PP_ITERATION_FINISH_2 >= 199
#        define MSGPACK_PP_ITERATION_2 199
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 200 && MSGPACK_PP_ITERATION_FINISH_2 >= 200
#        define MSGPACK_PP_ITERATION_2 200
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 201 && MSGPACK_PP_ITERATION_FINISH_2 >= 201
#        define MSGPACK_PP_ITERATION_2 201
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 202 && MSGPACK_PP_ITERATION_FINISH_2 >= 202
#        define MSGPACK_PP_ITERATION_2 202
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 203 && MSGPACK_PP_ITERATION_FINISH_2 >= 203
#        define MSGPACK_PP_ITERATION_2 203
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 204 && MSGPACK_PP_ITERATION_FINISH_2 >= 204
#        define MSGPACK_PP_ITERATION_2 204
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 205 && MSGPACK_PP_ITERATION_FINISH_2 >= 205
#        define MSGPACK_PP_ITERATION_2 205
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 206 && MSGPACK_PP_ITERATION_FINISH_2 >= 206
#        define MSGPACK_PP_ITERATION_2 206
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 207 && MSGPACK_PP_ITERATION_FINISH_2 >= 207
#        define MSGPACK_PP_ITERATION_2 207
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 208 && MSGPACK_PP_ITERATION_FINISH_2 >= 208
#        define MSGPACK_PP_ITERATION_2 208
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 209 && MSGPACK_PP_ITERATION_FINISH_2 >= 209
#        define MSGPACK_PP_ITERATION_2 209
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 210 && MSGPACK_PP_ITERATION_FINISH_2 >= 210
#        define MSGPACK_PP_ITERATION_2 210
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 211 && MSGPACK_PP_ITERATION_FINISH_2 >= 211
#        define MSGPACK_PP_ITERATION_2 211
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 212 && MSGPACK_PP_ITERATION_FINISH_2 >= 212
#        define MSGPACK_PP_ITERATION_2 212
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 213 && MSGPACK_PP_ITERATION_FINISH_2 >= 213
#        define MSGPACK_PP_ITERATION_2 213
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 214 && MSGPACK_PP_ITERATION_FINISH_2 >= 214
#        define MSGPACK_PP_ITERATION_2 214
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 215 && MSGPACK_PP_ITERATION_FINISH_2 >= 215
#        define MSGPACK_PP_ITERATION_2 215
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 216 && MSGPACK_PP_ITERATION_FINISH_2 >= 216
#        define MSGPACK_PP_ITERATION_2 216
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 217 && MSGPACK_PP_ITERATION_FINISH_2 >= 217
#        define MSGPACK_PP_ITERATION_2 217
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 218 && MSGPACK_PP_ITERATION_FINISH_2 >= 218
#        define MSGPACK_PP_ITERATION_2 218
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 219 && MSGPACK_PP_ITERATION_FINISH_2 >= 219
#        define MSGPACK_PP_ITERATION_2 219
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 220 && MSGPACK_PP_ITERATION_FINISH_2 >= 220
#        define MSGPACK_PP_ITERATION_2 220
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 221 && MSGPACK_PP_ITERATION_FINISH_2 >= 221
#        define MSGPACK_PP_ITERATION_2 221
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 222 && MSGPACK_PP_ITERATION_FINISH_2 >= 222
#        define MSGPACK_PP_ITERATION_2 222
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 223 && MSGPACK_PP_ITERATION_FINISH_2 >= 223
#        define MSGPACK_PP_ITERATION_2 223
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 224 && MSGPACK_PP_ITERATION_FINISH_2 >= 224
#        define MSGPACK_PP_ITERATION_2 224
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 225 && MSGPACK_PP_ITERATION_FINISH_2 >= 225
#        define MSGPACK_PP_ITERATION_2 225
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 226 && MSGPACK_PP_ITERATION_FINISH_2 >= 226
#        define MSGPACK_PP_ITERATION_2 226
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 227 && MSGPACK_PP_ITERATION_FINISH_2 >= 227
#        define MSGPACK_PP_ITERATION_2 227
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 228 && MSGPACK_PP_ITERATION_FINISH_2 >= 228
#        define MSGPACK_PP_ITERATION_2 228
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 229 && MSGPACK_PP_ITERATION_FINISH_2 >= 229
#        define MSGPACK_PP_ITERATION_2 229
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 230 && MSGPACK_PP_ITERATION_FINISH_2 >= 230
#        define MSGPACK_PP_ITERATION_2 230
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 231 && MSGPACK_PP_ITERATION_FINISH_2 >= 231
#        define MSGPACK_PP_ITERATION_2 231
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 232 && MSGPACK_PP_ITERATION_FINISH_2 >= 232
#        define MSGPACK_PP_ITERATION_2 232
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 233 && MSGPACK_PP_ITERATION_FINISH_2 >= 233
#        define MSGPACK_PP_ITERATION_2 233
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 234 && MSGPACK_PP_ITERATION_FINISH_2 >= 234
#        define MSGPACK_PP_ITERATION_2 234
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 235 && MSGPACK_PP_ITERATION_FINISH_2 >= 235
#        define MSGPACK_PP_ITERATION_2 235
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 236 && MSGPACK_PP_ITERATION_FINISH_2 >= 236
#        define MSGPACK_PP_ITERATION_2 236
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 237 && MSGPACK_PP_ITERATION_FINISH_2 >= 237
#        define MSGPACK_PP_ITERATION_2 237
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 238 && MSGPACK_PP_ITERATION_FINISH_2 >= 238
#        define MSGPACK_PP_ITERATION_2 238
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 239 && MSGPACK_PP_ITERATION_FINISH_2 >= 239
#        define MSGPACK_PP_ITERATION_2 239
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 240 && MSGPACK_PP_ITERATION_FINISH_2 >= 240
#        define MSGPACK_PP_ITERATION_2 240
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 241 && MSGPACK_PP_ITERATION_FINISH_2 >= 241
#        define MSGPACK_PP_ITERATION_2 241
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 242 && MSGPACK_PP_ITERATION_FINISH_2 >= 242
#        define MSGPACK_PP_ITERATION_2 242
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 243 && MSGPACK_PP_ITERATION_FINISH_2 >= 243
#        define MSGPACK_PP_ITERATION_2 243
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 244 && MSGPACK_PP_ITERATION_FINISH_2 >= 244
#        define MSGPACK_PP_ITERATION_2 244
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 245 && MSGPACK_PP_ITERATION_FINISH_2 >= 245
#        define MSGPACK_PP_ITERATION_2 245
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 246 && MSGPACK_PP_ITERATION_FINISH_2 >= 246
#        define MSGPACK_PP_ITERATION_2 246
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 247 && MSGPACK_PP_ITERATION_FINISH_2 >= 247
#        define MSGPACK_PP_ITERATION_2 247
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 248 && MSGPACK_PP_ITERATION_FINISH_2 >= 248
#        define MSGPACK_PP_ITERATION_2 248
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 249 && MSGPACK_PP_ITERATION_FINISH_2 >= 249
#        define MSGPACK_PP_ITERATION_2 249
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 250 && MSGPACK_PP_ITERATION_FINISH_2 >= 250
#        define MSGPACK_PP_ITERATION_2 250
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 251 && MSGPACK_PP_ITERATION_FINISH_2 >= 251
#        define MSGPACK_PP_ITERATION_2 251
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 252 && MSGPACK_PP_ITERATION_FINISH_2 >= 252
#        define MSGPACK_PP_ITERATION_2 252
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 253 && MSGPACK_PP_ITERATION_FINISH_2 >= 253
#        define MSGPACK_PP_ITERATION_2 253
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 254 && MSGPACK_PP_ITERATION_FINISH_2 >= 254
#        define MSGPACK_PP_ITERATION_2 254
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 255 && MSGPACK_PP_ITERATION_FINISH_2 >= 255
#        define MSGPACK_PP_ITERATION_2 255
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
#    if MSGPACK_PP_ITERATION_START_2 <= 256 && MSGPACK_PP_ITERATION_FINISH_2 >= 256
#        define MSGPACK_PP_ITERATION_2 256
#        include MSGPACK_PP_FILENAME_2
#        undef MSGPACK_PP_ITERATION_2
#    endif
# endif
#
# undef MSGPACK_PP_ITERATION_DEPTH
# define MSGPACK_PP_ITERATION_DEPTH() 1
#
# undef MSGPACK_PP_ITERATION_START_2
# undef MSGPACK_PP_ITERATION_FINISH_2
# undef MSGPACK_PP_FILENAME_2
#
# undef MSGPACK_PP_ITERATION_FLAGS_2
# undef MSGPACK_PP_ITERATION_PARAMS_2
