/*
Copyright Rene Rivera 2014
Distributed under the Boost Software License, Version 1.0.
(See accompanying file LICENSE_1_0.txt or copy at
http://www.boost.org/LICENSE_1_0.txt)
*/

#ifndef MSGPACK_PREDEF_DETAIL_PLAT_DETECTED
#define MSGPACK_PREDEF_DETAIL_PLAT_DETECTED 1
#endif
