#ifndef MATCHING_DISTANCE_DEF_DEBUG_H
#define MATCHING_DISTANCE_DEF_DEBUG_H

//#define MD_EXPERIMENTAL_TIMING
//#define MD_PRINT_HEAT_MAP
//#define MD_DEBUG
//#define MD_DO_CHECKS
//#define MD_DO_FULL_CHECK

#endif //MATCHING_DISTANCE_DEF_DEBUG_H
