#ifndef HOMCLOUD_CGAL
#define HOMCLOUD_CGAL

#include <CGAL/version_macros.h>
#if (CGAL_VERSION_MAJOR >= 4 && CGAL_VERSION_MINOR >= 11) || (CGAL_VERSION_MAJOR > 4)
#define CGAL_NEWER_API_4_11
#endif


#endif
